<?php
namespace Zeedhi\Queue;

use Stomp\Transport\Frame;

abstract class Consumer {

    /** @var string */
    protected static $queueName = null;
    /** @var Connection */
    protected $connection;
    /** @var array */
    protected $headers = array();

    public function __construct($connection) {
        $this->connection = $connection;
        if (static::$queueName === null) {
            throw Exception::missingConsumerQueueName(get_class($this));
        }
    }

    protected function nack() {
        throw Exception::nack();
    }

    /**
     * @param string $message
     *
     * @return void
     */
    abstract public function consume($message);

    protected function consumeFrame(Frame $frame) {
        try {
            $return = $this->consume($frame->body);
            $this->connection->ackFrame($frame);
            if ($return[0]) {
                foreach ($return[2] as $pid) {
                    if ($pid != $return[1])
                        $this->kill($pid);
                }
                $this->kill($return[1]);
            }
        } catch (Exception $e) {
            $this->connection->nackFrame($frame);
            if ($e->getMessage() !== "NACK") {
                throw $e;
            }
        }
    }

    protected function consumeFrameAndFinish(Frame $frame) {
        try {
            $return = $this->consume($frame->body);
            $this->connection->ackFrame($frame);
        } catch (Exception $e) {
            $this->connection->nackFrame($frame);
            if ($e->getMessage() !== "NACK") {
                throw $e;
            }
        }
    }

    public function run() {
        $this->connection->subscribe(static::$queueName, uniqid(get_class($this)), 'client-individual', $this->headers);
        while (true) {
            if ($frame = $this->connection->waitForFrame()) {
                $this->consumeFrame($frame);
            }
        }
    }
    public function runAndFinish() {
        $this->connection->subscribe(static::$queueName, uniqid(get_class($this)), 'client-individual', $this->headers);
        if ($frame = $this->connection->waitForFrame()) {
            $this->consumeFrameAndFinish($frame);
        }
    }
    
    function kill($pid){
        return stripos(php_uname('s'), 'win')>-1  ? exec("taskkill /F /T /PID $pid") : exec("kill -9 $pid");
    }
}
