<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 28/10/14
 * Time: 14:06
 */

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\ParameterBag;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\FPA;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;
use HCM\FPA\Repository\ExportacaoExcelRepository;


class ExportacaoExcelService {

    const XLS   = '001';
    const XLSX  = '002';

    const IN  = '101';
    const OUT = '102';

    const CARACTER_ACENTUADO =     'ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõöøùúûýýþÿ';
    const CARACTER_NAO_ACENTUADO = 'AAAAAAACEEEEIIIIDNOOOOOOUUUUYBSAAAAAAACEEEEIIIIDNOOOOOOUUUYYBY';
    const APENAS_CARACTERES_ALFANUMERICOS = '/[^a-zA-Z0-9\s]/';
    const CARACTER_BRANCO = ' ';
    const CARACTER_VAZIO = '';


    public function __construct (EntityManager $entityManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ExportacaoExcelRepository $exportacaoexcelrepository){
        $this->entityManager = $entityManager;
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->ExportacaoExcelRepository = $exportacaoexcelrepository;
    }

    /**
     * Executa a criação do template e faz o download do mesmo
     * @param array $info
     * @param array $eventos
     * @param int $nrorg
     * @throws Exception
     * @throws \Exception
     */
    public function gravar($row){
        try {
            $file = $this->makeCode($row, $row['NREVENTOM']);
            return $this->download($file, $row['NMESTRUTGEREN']);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere a estrutura no arquivo
     * @param array $info
     * @param array $eventos 
     * @throws \Exception
     */
    private function makeCode($info, $eventos){
        try {
            $file = new \PHPExcel();
            $file->setActiveSheetIndex(0);
//            $file->getActiveSheet()->getStyle('A1:BA100000')->getFill()->applyFromArray(array('type' => \PHPExcel_Style_Fill::FILL_SOLID, 'startcolor' => array('rgb' => "FFFFFF")));
            $this->makehead($file, $info['NMESTRUTLEGAL'], $info['NMESTRUTGEREN']);
            $colLivre = $this->makeColHead($file);
            $this->makeColEventos($file, $colLivre, $eventos, $info['NMEVENTOH']);
            $query=$this->ExportacaoExcelRepository->getVinculos($info);
            $this->makeRowInfo($file, $info, $query);

            return $file;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Adiciona os dados de cabeçalho do Excel
     *
     * @param \PHPExcel $file
     * @param String $nmestrutlegal
     * @param String $nmestrutgeren
     * @throws \Exception
     */
    private function makeHead($file, $nmestrutlegal, $nmestrutgeren){
        try {
            $file->getActiveSheet()->setCellValue('B1', 'Estrutura Legal');
            $file->getActiveSheet()->getStyle('B1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $file->getActiveSheet()->setCellValue('C1', $nmestrutlegal);
            $file->getActiveSheet()->getStyle('C1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);

            $file->getActiveSheet()->setCellValue('B2', 'Estrutura Gerencial');
            $file->getActiveSheet()->getStyle('B2')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $file->getActiveSheet()->setCellValue('C2', $nmestrutgeren);
            $file->getActiveSheet()->getStyle('C2')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);

            $file->getActiveSheet()->setCellValue('B3', 'Competencia');
            $file->getActiveSheet()->getStyle('B3')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $file->getActiveSheet()->setCellValue('C3', '##/##/####');
            $file->getActiveSheet()->getStyle('C3')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);

            $file->getActiveSheet()->getColumnDimension('B')->setAutoSize(true);
            $file->getActiveSheet()->getColumnDimension('C')->setAutoSize(true);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere o cabeçalho das colunas
     *
     * @param \PHPExcel $file
     *
     * @return String;
     *
     * @throws \Exception
     */
    private function makeColHead($file){
        try {
            $colshead = array(
                'Vínc.',
                'Vínculo',
                'Situação Funcional',
                'Admissão',
                'Cargo',
                'Estrut. Gerencial');

            $colId = 'A';
            $rowId = 6;

            foreach ($colshead as $colhead) {
                $file->getActiveSheet()->setCellValue($colId . $rowId, $colhead);
                $file->getActiveSheet()->getStyle($colId . $rowId)->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
                $file->getActiveSheet()->getColumnDimension($colId++)->setAutoSize(true);
            }

            return $colId;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     *
     * @param \PHPExcel $file
     * @param String $colIniEventos
     * @param array $eventos
     * @throws \Exception
     */
    private function makeColEventos($file, $colIniEventos, $eventos, $nome){
        try {
            $colId = $colIniEventos;
            $rowId = 5;
            $rowId2 = 6;
            $cont = 0;
            foreach ($eventos as $evento) {
                $ev = strval($evento);
                $file->getActiveSheet()->setCellValue($colId . $rowId, $ev);
                $file->getActiveSheet()->getStyle($colId . $rowId)->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
                $file->getActiveSheet()->setCellValue($colId . $rowId2, $nome[$cont]);
                $file->getActiveSheet()->getStyle($colId . $rowId2)->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $colId++;
                $cont++;
            }

            $file->getActiveSheet()->setCellValue($colId . $rowId2, 'Observação');
            $file->getActiveSheet()->getStyle($colId . $rowId2)->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $file->getActiveSheet()->getColumnDimension($colId )->setAutoSize(true);
            $colId++;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param \PHPExcel $file
     * @param array $infos
     * @throws \Exception
     */
    private function makeRowInfo( $file, $infos, $query)
    {
        try {
            $rowId = 7;
            foreach ($query as $info) {
                $colId = 'A';
                $file->getActiveSheet()->setCellValue($colId . $rowId, $info['NRVINCULOM']);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $colId++;
                $file->getActiveSheet()->setCellValue($colId . $rowId, $info['NMVINCULOM']);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $colId++;
                $file->getActiveSheet()->setCellValue($colId . $rowId, $info['NMSITUFUNCH']);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $colId++;
                $file->getActiveSheet()->setCellValue($colId . $rowId, $info['DTADMISSAOVINC']);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $colId++;
                $file->getActiveSheet()->setCellValue($colId . $rowId, $info['NMOCUPACAOH']);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $colId++;
                $file->getActiveSheet()->setCellValue($colId . $rowId, $info['NMESTRUTURAM']);
                $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                $rowId++;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param String $string
     * @return mixed
     */
    private static function _removeCaracteresEspeciais($string)
    {
        $string = preg_replace(self::APENAS_CARACTERES_ALFANUMERICOS, self::CARACTER_VAZIO, $string);
        $string = str_replace('  ', ' ',$string);

        return $string;
    }

    /**
     * @param \PHPExcel $file
     * @param int $nrestrutgeren
     * @param int $nrorg
     * @throws Exception
     */
    private function download($file, $nrestrutgeren)
    {
        try {
            $nomeArquivo = "Formulario_";
            $nomeArquivo .= self::_removeCaracteresEspeciais($nrestrutgeren);
            $nomeArquivo .= '.xls';
            $expDir = FPA::getTempDirectory();
            /*$expDir = 'APP_PATH' . '/uploads/' . $nrorg . '/Exportacao/';
            if (!file_exists($expDir)) {
                mkdir($expDir, 0777, true);
            }
            $expDir = realpath($expDir); */
            
            $write = \PHPExcel_IOFactory::createWriter($file, 'Excel5');
            $write->save($expDir . DIRECTORY_SEPARATOR . $nomeArquivo);

            $file = ($expDir . DIRECTORY_SEPARATOR . $nomeArquivo);
            return $file;
            //$file->
            //download($file);
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /********************************************* IMPORTAÇÃO *******************************************/

    /**
     * CSU010 - Preparação da informação.
     *
     * @param String  $caminho
     *
     * @return array
     * @throws \Exception
     */
    public function preparaInformacao( $caminho)
    {
        try {
            $file = \PHPExcel_IOFactory::load($caminho);
            $file->setActiveSheetIndex(0);

            $cabecalho = $this->preparaCabecalho($file);
            $eventos = $this->preparaEventos($file);
            $funcs = $this->preparaFunc($file);
            $info = $this->preparaInfo($file, $funcs, $eventos, $cabecalho);

            return array(
                'CABECALHO' => $cabecalho,
                'EVENTO' => $eventos,
                'FUNCIONARIO' => $funcs,
                'INFO' => $info
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * CSU010a - Captura as informações do cabeçalho e retorna um array.
     *
     * @param \PHPExcel $file
     *
     * @return array
     * @throws \Exception
     */
    private function preparaCabecalho( $file)
    {
        try {
            $nmestrutlegal = $file->getActiveSheet()->getCell('C1')->getValue();
            $nmestrutgeren = $file->getActiveSheet()->getCell('C2')->getValue();

            $dtmescompetencia = $file->getActiveSheet()->getCell('C3')->getValue();
            $dtmescompetencia = DateUtil::getDataDeString(jdtojulian($dtmescompetencia + 2415032), DateUtil::FORMATO_AMERICANO_2, true);
//        $dtmescompetencia = $dtmescompetencia->format('d/m/Y');

            return array('NMESTRUTLEGAL' => $nmestrutlegal,
                'NMESTRUTGEREN' => $nmestrutgeren,
                'DTMESCOMPETENCIA' => $dtmescompetencia);
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * CSU010b - Captura os eventos.
     *
     * @param \PHPExcel $file
     *
     * @return array
     * @throws \Exception
     */
    private function preparaEventos( $file)
    {
        try {$colId  = 'G';
            $rowId  = 5;
            $rowId2 = 6;
            $eventos = array();

            $nrevento = $file->getActiveSheet()->getCell($colId.$rowId)->getValue();
            $nmevento = $file->getActiveSheet()->getCell($colId++.$rowId2)->getValue();
            while(!empty($nrevento) && !empty($nmevento)){
                $eventos[] = array('CODIGO' => $nrevento, 'EVENTO' => $nmevento);
                $nrevento = $file->getActiveSheet()->getCell($colId.$rowId)->getValue();
                $nmevento = $file->getActiveSheet()->getCell($colId++.$rowId2)->getValue();
            }

            return $eventos;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * CSU010c - Constrói o array com a informação dos eventos
     * @param \PHPExcel $file
     * @return array
     * @throws \Exception
     */
    private function preparaFunc( $file)
    {
        try {
            $colId = 'A';
            $colId2 = 'B';
            $colId3 = 'C';
            $colId4 = 'D';
            $colId5 = 'E';
            $rowId = 7;

            $funcs = array();

            $idFunc = $file->getActiveSheet()->getCell($colId . $rowId)->getValue();
            $func = $file->getActiveSheet()->getCell($colId2 . $rowId)->getValue();
            $situFunc = $file->getActiveSheet()->getCell($colId3 . $rowId)->getValue();
            $admin = $file->getActiveSheet()->getCell($colId4 . $rowId)->getValue();
            $cargo = $file->getActiveSheet()->getCell($colId5 . $rowId++)->getValue();

            while (!empty($idFunc)) {

                $funcs[] = array(
                    'CODIGO' => $idFunc,
                    'FUNCIONARIO' => $func,
                    'SITUFUNC' => $situFunc,
                    'ADMISSAO' => $admin,
                    'CARGO' => $cargo
                );

                $idFunc = $file->getActiveSheet()->getCell($colId . $rowId)->getValue();
                $func = $file->getActiveSheet()->getCell($colId2 . $rowId)->getValue();
                $situFunc = $file->getActiveSheet()->getCell($colId3 . $rowId)->getValue();
                $admin = $file->getActiveSheet()->getCell($colId4 . $rowId)->getValue();
                $cargo = $file->getActiveSheet()->getCell($colId5 . $rowId++)->getValue();

            }

            return $funcs;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    /**
     * CSU010d - Constrói o array com a informação dos eventos
     *
     * @param \PHPExcel $file
     * @param array $funcs
     * @param array $eventos
     * @param array $cabeca
     *
     * @return array
     * @throws \Exception
     */
    private function preparaInfo( $file, $funcs, $eventos, $cabeca)
    {
        try {
//          $colId      = 'F';
            $rowId = 7;
            $tamFuncs = count($funcs);
            $tamEventos = count($eventos);

            $info = array();

            for ($c = 0; $c < $tamFuncs; $rowId++, $c++) {
                $colId = 'G';
                for ($d = 0; $d < $tamEventos; $colId++, $d++) {
                    $info[] = array(
                        'CODIGO' => $funcs[$c]['CODIGO'],
                        'DTMESCOMPETENCIA' => $cabeca['DTMESCOMPETENCIA'],
                        'EVENTO' => $eventos[$d]['CODIGO'],
                        'VALOR'  => $file->getActiveSheet()->getCell($colId.$rowId)->getValue()
                    );
                }
            }
            return $info;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function preparaInfoCtbl($file) {
        try {
            // Definição dos Id's das colunas do xls
            $colId = 'A';
            $colId2 = 'B';
            $colId3 = 'C';
            $colId4 = 'D';
            $colId5 = 'E';
            $colId6 = 'F';
            $colId7 = 'G';
            $colId8 = 'H';
            $colId9 = 'I';
            $colId10 = 'J';
            $colId11 = 'K';
            $colId12 = 'L';
            $rowId = 2;
            
            $evCtbl = array();
            
            $org               = $file->getActiveSheet()->getCell($colId   . $rowId)->getValue();
            $evento            = $file->getActiveSheet()->getCell($colId2  . $rowId)->getValue();
            $ctblDeb           = $file->getActiveSheet()->getCell($colId5  . $rowId)->getValue();
            $descCrblDeb       = $file->getActiveSheet()->getCell($colId6  . $rowId)->getValue();
            $cdHistPadrDeb     = $file->getActiveSheet()->getCell($colId7  . $rowId)->getValue();
            $descHistPadrDeb   = $file->getActiveSheet()->getCell($colId8  . $rowId)->getValue();
            $ctblCre           = $file->getActiveSheet()->getCell($colId9  . $rowId)->getValue();
            $descCrblCre       = $file->getActiveSheet()->getCell($colId10 . $rowId)->getValue();
            $cdHistPadrCre     = $file->getActiveSheet()->getCell($colId11 . $rowId)->getValue();
            $descHistPadrCre   = $file->getActiveSheet()->getCell($colId12 . $rowId++)->getValue();
            
            // Seta o array com os dados do xls
            while (!empty($evento)) {
                
                $evCtbl[] = array(
                    'ORGANIZACAO'  => $org,
                    'EVENTO'       => $evento,
                    'CONTDEBITO'   => $ctblDeb,
                    'DESCDEBITO'   => $descCrblDeb,
                    'HISTPDRDEB'   => $cdHistPadrDeb,
                    'DSCHPDRDEB'   => $descHistPadrDeb,
                    'CONTCREDIT'   => $ctblCre,
                    'DESCCREDIT'   => $descCrblCre,
                    'HISTPDRCRE'   => $cdHistPadrCre,
                    'DSCHPDRCRE'   => $descHistPadrCre
                );
                
                $org               = $file->getActiveSheet()->getCell($colId   . $rowId)->getValue();
                $evento            = $file->getActiveSheet()->getCell($colId2  . $rowId)->getValue();
                $ctblDeb           = $file->getActiveSheet()->getCell($colId5  . $rowId)->getValue();
                $descCrblDeb       = $file->getActiveSheet()->getCell($colId6  . $rowId)->getValue();
                $cdHistPadrDeb     = $file->getActiveSheet()->getCell($colId7  . $rowId)->getValue();
                $descHistPadrDeb   = $file->getActiveSheet()->getCell($colId8  . $rowId)->getValue();
                $ctblCre           = $file->getActiveSheet()->getCell($colId9  . $rowId)->getValue();
                $descCrblCre       = $file->getActiveSheet()->getCell($colId10 . $rowId)->getValue();
                $cdHistPadrCre     = $file->getActiveSheet()->getCell($colId11 . $rowId)->getValue();
                $descHistPadrCre   = $file->getActiveSheet()->getCell($colId12 . $rowId++)->getValue();
            }
            return $evCtbl;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function preparaInfoEvenDePara($file) {
        try {
            // Definição dos Id's das colunas do xls
            $colId = 'A';
            $colId2 = 'B';
            $colId3 = 'C';
            $colId4 = 'D';
            $colId5 = 'E';
            $colId6 = 'F';
            $rowId = 2;
            $max = 0;
            $evDePara = array();
            $chegouFinal = false;
            
            $dtinicio                = $file->getActiveSheet()->getCell($colId   . $rowId)->getValue();
            $tpevento                = $file->getActiveSheet()->getCell($colId2  . $rowId)->getValue();
            $dtcompetencia           = $file->getActiveSheet()->getCell($colId3  . $rowId)->getValue();
            $nmevento                = $file->getActiveSheet()->getCell($colId4  . $rowId)->getValue();
            $nrpara                  = $file->getActiveSheet()->getCell($colId5  . $rowId)->getValue();
            $nrde                    = $file->getActiveSheet()->getCell($colId6  . $rowId++)->getValue();
            
            // Seta o array com os dados do xls
            while ($max < 5) {
                if(empty($nrde) && empty($nrpara)){
                    $chegouFinal = true;
                    $max = 6;
                }else if(empty($nrde)){
                    $max ++;
                    
                    $dtinicio                = $file->getActiveSheet()->getCell($colId   . $rowId)->getValue();
                    $tpevento                = $file->getActiveSheet()->getCell($colId2  . $rowId)->getValue();
                    $dtcompetencia           = $file->getActiveSheet()->getCell($colId3  . $rowId)->getValue();
                    $nmevento                = $file->getActiveSheet()->getCell($colId4  . $rowId)->getValue();
                    $nrpara                  = $file->getActiveSheet()->getCell($colId5  . $rowId)->getValue();
                    $nrde                    = $file->getActiveSheet()->getCell($colId6  . $rowId++)->getValue();
                    
                }else{
                    $max = 0;
                    
                    $evDePara[] = array(
                        'DTINICIO'       => $dtinicio,
                        'TPEVENTO'       => $tpevento,
                        'DTMESCOMPETENC' => $dtcompetencia,
                        'NMEVENTOH'      => $nmevento, 
                        'NRPARA'         => $nrpara, 
                        'NRDE'           => $nrde
                    );
                    
                    $dtinicio                = $file->getActiveSheet()->getCell($colId   . $rowId)->getValue();
                    $tpevento                = $file->getActiveSheet()->getCell($colId2  . $rowId)->getValue();
                    $dtcompetencia           = $file->getActiveSheet()->getCell($colId3  . $rowId)->getValue();
                    $nmevento                = $file->getActiveSheet()->getCell($colId4  . $rowId)->getValue();
                    $nrpara                  = $file->getActiveSheet()->getCell($colId5  . $rowId)->getValue();
                    $nrde                    = $file->getActiveSheet()->getCell($colId6  . $rowId++)->getValue();
                    
                }
            }
            if(!$chegouFinal && $max == 5){
                return false;
            }else{
                return $evDePara;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function preparaInfoLoteFicha($file) {
        try {
            // Definição dos Id's das colunas do xls
            $colId = 'A';
            $colId2 = 'B';
            $colId3 = 'C';
            $colId4 = 'D';
            $colId5 = 'E';
            $colId6 = 'F';
            $colId7 = 'G';
            $rowId = 2;
            $max = 0;
            $itensLote = array();
            $chegouFinal = false;
            
            $tpmoviment                = $file->getActiveSheet()->getCell($colId   . $rowId)->getValue();
            $nrvinculom                = $file->getActiveSheet()->getCell($colId2  . $rowId)->getValue();
            $nreventom                 = $file->getActiveSheet()->getCell($colId3  . $rowId)->getValue();
            //$dtocorrencia              = $file->getActiveSheet()->getCell($colId4  . $rowId)->getValue();
            if (gettype($file->getActiveSheet()->getCell($colId4  . $rowId)->getValue()) === "string") {
                $dtocorrencia = $file->getActiveSheet()->getCell($colId4  . $rowId)->getValue();
            } else {
                $dtocorrencia = date('d/m/Y', \PHPExcel_Shared_Date::ExcelToPHP($file->getActiveSheet()->getCell($colId4  . $rowId)->getValue()));
            }
            $vreventofolha             = $file->getActiveSheet()->getCell($colId5  . $rowId)->getValue();
            $vrreffolha                = $file->getActiveSheet()->getCell($colId6  . $rowId)->getValue();
            $dscalculo                 = $file->getActiveSheet()->getCell($colId7  . $rowId++)->getValue();
            
            // Seta o array com os dados do xls
            while ($max < 10) {
                if(empty($nrvinculom) && empty($nreventom)){
                    $max ++;
                    
                    $tpmoviment                = $file->getActiveSheet()->getCell($colId   . $rowId)->getValue();
                    $nrvinculom                = $file->getActiveSheet()->getCell($colId2  . $rowId)->getValue();
                    $nreventom                 = $file->getActiveSheet()->getCell($colId3  . $rowId)->getValue();
                    //$dtocorrencia              = $file->getActiveSheet()->getCell($colId4  . $rowId)->getValue();
                    if (gettype($file->getActiveSheet()->getCell($colId4  . $rowId)->getValue()) === "string") {
                        $dtocorrencia = $file->getActiveSheet()->getCell($colId4  . $rowId)->getValue();
                    } else {
                        $dtocorrencia = date('d/m/Y', \PHPExcel_Shared_Date::ExcelToPHP($file->getActiveSheet()->getCell($colId4  . $rowId)->getValue()));
                    }
                    $vreventofolha             = $file->getActiveSheet()->getCell($colId5  . $rowId)->getValue();
                    $vrreffolha                = $file->getActiveSheet()->getCell($colId6  . $rowId)->getValue();
                    $dscalculo                 = $file->getActiveSheet()->getCell($colId7  . $rowId++)->getValue();
                    
                }else{
                    $max = 0;
                    
                    $itensLote[] = array(
                                            'NRTPMOVIMENT'   => $tpmoviment,
                                            'NRVINCULOM'     => $nrvinculom,
                                            'NREVENTOM'      => $nreventom,
                                            'DTOCORRENCIA'   => $dtocorrencia,
                                            'VREVENTOFOLHA'  => $vreventofolha, 
                                            'VRREFFOLHA'     => $vrreffolha, 
                                            'DSCALCULO'      => $dscalculo
                                        );
                    
                    $tpmoviment                = $file->getActiveSheet()->getCell($colId   . $rowId)->getValue();
                    $nrvinculom                = $file->getActiveSheet()->getCell($colId2  . $rowId)->getValue();
                    $nreventom                 = $file->getActiveSheet()->getCell($colId3  . $rowId)->getValue();
                    //$dtocorrencia              = $file->getActiveSheet()->getCell($colId4  . $rowId)->getValue();
                    if (gettype($file->getActiveSheet()->getCell($colId4  . $rowId)->getValue()) === "string") {
                        $dtocorrencia = $file->getActiveSheet()->getCell($colId4  . $rowId)->getValue();
                    } else {
                        $dtocorrencia = date('d/m/Y', \PHPExcel_Shared_Date::ExcelToPHP($file->getActiveSheet()->getCell($colId4  . $rowId)->getValue()));
                    }
                    $vreventofolha             = $file->getActiveSheet()->getCell($colId5  . $rowId)->getValue();
                    $vrreffolha                = $file->getActiveSheet()->getCell($colId6  . $rowId)->getValue();
                    $dscalculo                 = $file->getActiveSheet()->getCell($colId7  . $rowId++)->getValue();
                    
                }
            }
            
            return $itensLote;
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function preparaInfoEsoBxEsocial($file) {
        try {
            // Definição dos Id's das colunas do xls
            $colId = 'A';
            $colId2 = 'B';
            $colId3 = 'C';
            $rowId = 2;
            $max = 0;
            $esoBxEsocial = array();
            $chegouFinal = false;
            
            $nrcpftrabalhador = $file->getActiveSheet()->getCell($colId   . $rowId)->getValue();
            if (gettype($file->getActiveSheet()->getCell($colId2  . $rowId)->getValue()) === "string") {
                $dtiniperiodo = $file->getActiveSheet()->getCell($colId2  . $rowId)->getValue();
            } else {
                $dtiniperiodo = date('d/m/Y', \PHPExcel_Shared_Date::ExcelToPHP($file->getActiveSheet()->getCell($colId2  . $rowId)->getValue()));
            }
            if (gettype($file->getActiveSheet()->getCell($colId3  . $rowId)->getValue()) === "string") {
                $dtfimperiodo = $file->getActiveSheet()->getCell($colId3  . $rowId++)->getValue();
            } else {
                $dtfimperiodo = date('d/m/Y', \PHPExcel_Shared_Date::ExcelToPHP($file->getActiveSheet()->getCell($colId3  . $rowId++)->getValue()));
            }
            
            // Seta o array com os dados do xls
            while (!empty($nrcpftrabalhador)) {
                $esoBxEsocial[] = array(
                    'NRCPFTRABALHADOR' => $nrcpftrabalhador,
                    'DTINIPERIODO'     => $dtiniperiodo,
                    'DTFIMPERIODO'     => $dtfimperiodo,
                );
                
                $nrcpftrabalhador = $file->getActiveSheet()->getCell($colId   . $rowId)->getValue();
                $dtiniperiodo     = $file->getActiveSheet()->getCell($colId2  . $rowId)->getValue();
                if (gettype($file->getActiveSheet()->getCell($colId2  . $rowId)->getValue()) === "string") {
                    $dtiniperiodo = $file->getActiveSheet()->getCell($colId2  . $rowId)->getValue();
                } else {
                    $dtiniperiodo = date('d/m/Y', \PHPExcel_Shared_Date::ExcelToPHP($file->getActiveSheet()->getCell($colId2  . $rowId)->getValue()));
                }
                if (gettype($file->getActiveSheet()->getCell($colId3  . $rowId)->getValue()) === "string") {
                    $dtfimperiodo = $file->getActiveSheet()->getCell($colId3  . $rowId++)->getValue();
                } else {
                    $dtfimperiodo = date('d/m/Y', \PHPExcel_Shared_Date::ExcelToPHP($file->getActiveSheet()->getCell($colId3  . $rowId++)->getValue()));
                }
            };
            
            return $esoBxEsocial;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /* Adiquiri as informações de uma planilha de layout simples. A 1ª linha é o
     * cabeçalho, este busca dados da coluna 'colSart' até o comprimento dos
     * registros em 'head', começando da linha 'linStart' em diante até que o
     * dado de indice 'headPrincipal' seja vazio. Os índices dos arrays são
     * dados por 'head'.
     */
    public function preparaInfo_generic($file, $head, $headPrincipal, $linStart = 2, $colStart = 'A') {
        try {
            $dados = array();
            //percorre todas as linhas enquanto $headPrincipal estiver preenchido
            for($lin = $linStart; true; $lin++) {
                //inicializa coluna e array da linha
                $col = $colStart;
                $dadosLin = array();
                //captura todos os dados de uma linha de acordo com $head
                foreach($head as $h)
                    $dadosLin[$h] = $file->getActiveSheet()->getCell($col++.$lin)->getValue();
                //Quando a dado do índice $headerPrincipal for vazio este para o processamento
                if(empty($dadosLin[$headPrincipal]))
                    break;
                //Cria nova linha com as colunas exportadas
                $dados[] = $dadosLin;
            }
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /********************************************************************** Geração Template Geral **********************************************************************/
    
    public function gravarTemplate($eventos, $nrorg, $cabecalho, $nomeOperacao) {
        try {
            $file = $this->makeCodeTemplate($eventos, $cabecalho);
            return $this->downloadTemplate($file, $nomeOperacao, $nrorg);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function makeCodeTemplate($eventos, $cabecalho) {
        try {
            $file = new \PHPExcel();
            $file->setActiveSheetIndex(0);
            $colLivre = $this->fazerCabecalhoTemplate($file, $cabecalho);
            $this->makeRowInfoTemplate($file, $eventos);

            return $file;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function fazerCabecalhoTemplate($file, $cabecalho) {
        try {
            $colId = 'A';
            $rowId = 1;

            foreach ($cabecalho as $coluna) {
                $file->getActiveSheet()->setCellValue($colId . $rowId, $coluna);
                $file->getActiveSheet()->getStyle($colId . $rowId)->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
                $file->getActiveSheet()->getColumnDimension($colId++)->setAutoSize(true);
            }

            return $colId;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function downloadTemplate($file, $nomeOperacao, $nrorg) {
        try {
            $nomeArquivo = "Formulario_";
            $nomeArquivo .= self::_removeCaracteresEspeciais($nomeOperacao);
            $nomeArquivo .= '.xls';
            
            $expDir = FPA::getTempDirectory();
            $write = \PHPExcel_IOFactory::createWriter($file, 'Excel5');
            $write->save($expDir . DIRECTORY_SEPARATOR . $nomeArquivo);
            
            $file = ($expDir . DIRECTORY_SEPARATOR . $nomeArquivo);
            return $file;
            
        } catch (\Exception $e) {
            throw new \Exception("Erro ao realizar a exportação");
        }

    }
    
    private function makeRowInfoTemplate($file, $infos) {
        try {
            $rowId = 2;
            foreach ($infos as $info) {
                $colId = 'A';
                
                foreach ($info as $dado) {
                    $file->getActiveSheet()->setCellValue($colId . $rowId, $dado);
                    $file->getActiveSheet()->getColumnDimension($colId)->setAutoSize(true);
                    $colId++;
                }
                $rowId++;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /*** Geração Template Simples - Apenas cabeçalho ********************************************************************************************************************/
    
    /*Gera uma planilha xls template com o nome e o cabeçalho dado.
     * O cabeçalho é um array que será posto na linha 1, ocupando quantas colunas forem necesários.
     * É possível gerar um conteúdo (pode-se usar dados reais ou apenas por exemplo). O conteúdo é
     * um array de array, sendo o primeiro array de dados das linhas (a partir da linha 2) e o
     * outro array interno é as colunas.
     */
    public function gravarTemplate_1($nrorg, $cabeçalho, $nomeArquivo, $conteudo = null) {
        $file = new \PHPExcel();
        $file->setActiveSheetIndex(0);
        
        //Monta o cabeçalho
        $this->makeOneLine($file->getActiveSheet(), 1, $cabeçalho, true, true);
        
        //Monta o conteúdo, opcional.
        if(is_array($conteudo)) {
            $line = 2;
            foreach($conteudo as $row)
                $this->makeOneLine($file->getActiveSheet(), $line++, $row);
        }
        
        return $this->downloadTemplate($file, $nomeArquivo, $nrorg);
    }
    
    /*Monta apenas uma linha.
     * $sheet       planilha activa em uso (activeSheet)
     * $line        número da linha que será montado
     * $data        array com o dado a ser montado
     * $border      quando verdadeiro uma borda é feita
     * $autoSize    quando verdadeiro o tamanho da coluna será automatico
     */
    private function makeOneLine($sheet, $line, $data, $border = false, $autoSize = false) {
        $col = 'A';
        foreach($data as $value) {
            $sheet->setCellValue($col.$line, $value);
            if($border)
                $sheet->getStyle($col.$line)->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            if($autoSize)
                $sheet->getColumnDimension($col)->setAutoSize(true);
            ++$col;
        }
    }
}