<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;

class SituacaoFrequenciaController extends Crud {

    protected $dataSourceGetSituacaoFrequencia      = 'ptoSituFreqParam_getSituacaoFrequencia';
    protected $dataSourceGetParamSituacaoFrequencia = 'ptoSituFreqParam_getParametrosSituacaoFrequencia';
    protected $dataSourceSetSituacaoFrequenciaParam = 'ptoSituFreqParam_setSituacaoFrequencia';
    protected $dataSourceSetParamSituacaoFrequencia = 'ptoSituFreqParam_setParametrosSituacaoFrequencia';
    protected $dataSourceSetSituacaoFrequencia      = 'ptoSituFrequencia_setSituacaoFrequencia';


    protected $parameterBagColumns = array('NRORG', 'NRORGPADRAO', 'DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag){
        $this->parameterBag = $parameterBag;
        parent::__construct($dataSourceManager);
    }

    public function getSituacaoFrequencia (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetSituacaoFrequencia);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function getParametrosSituacaoFrequencia (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetParamSituacaoFrequencia);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setSituacaoEParametrosFrequencia (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetSituacaoFrequencia;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRSITUFREQPARAM'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORGPADRAO'], 'PTO_SITUFREQPARAM', 1, 12, true);
                    $row['NRSITUFREQUENCIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORGTRAB'], 'PTO_SITUFREQUENCIA', 1, 12, true);
                    $row['NRORG'] = $row['NRORGTRAB'];
                } else {
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }
                
                if(!array_key_exists('DTFIMVIGENCIA', $row)){
                    $row['DTFIMVIGENCIA'] = null;
                }

                if(!array_key_exists('NRADVERTSITUFREQ', $row)){
                    $row['NRADVERTSITUFREQ'] = null;
                }

                if(!array_key_exists('NREVENTOM', $row)){
                    $row['NREVENTOM'] = null;
                }
            }
            parent::save($request, $response);
            $this->dataSourceName = $this->dataSourceSetSituacaoFrequenciaParam;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setSituacaoFrequencia (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetSituacaoFrequencia;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRSITUFREQUENCIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORGTRAB'], 'PTO_SITUFREQUENCIA', 1, 12, true);
                    $row['NRORG'] = $row['NRORGTRAB'];
                } else {
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }
                
                if(!array_key_exists('DTFIMVIGENCIA', $row)){
                    $row['DTFIMVIGENCIA'] = null;
                }

                if(!array_key_exists('NRADVERTSITUFREQ', $row)){
                    $row['NRADVERTSITUFREQ'] = null;
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setParametrosSituacaoFrequencia (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetParamSituacaoFrequencia;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === true){
                    $row['DTINCLUSAO']      = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO']  = $row['CDOPERADOR_LOG'];
                    $row['NRORGINCLUSAO']   = $row['NRORGTRAB'];
                    $row['DTULTATU']        = $row['DTATUAL'];
                    $row['CDOPERULTATU']    = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU']     = $row['NRORGTRAB'];
                    $row['NRSITUFREQPARAM'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORGPADRAO'], 'PTO_SITUFREQPARAM', 1, 12, true);
                    // $row['NRSITUFREQUENCIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORGTRAB'], 'PTO_SITUFREQUENCIA', 1, 12, true);
                    $row['NRORG']           = $row['NRORGTRAB'];
                } else {
                    $row['DTULTATU']        = $row['DTATUAL'];
                    $row['CDOPERULTATU']    = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU']     = $row['NRORGTRAB'];
                }

                if(!array_key_exists('NREVENTOM', $row)){
                    $row['NREVENTOM'] = null;
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteSituacaoEParametroFrequencia (DTO\Request\Row $row, DTO\Response $response) {
        try{
            $dataSet = new DataSet($this->dataSourceSetSituacaoFrequenciaParam, array($row->getRow()));
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $dataSet = new DataSet($this->dataSourceSetSituacaoFrequencia, array($row->getRow()));
            $deletedRows = $this->dataSourceManager->delete($dataSet);

            $response->addDataSet(new DataSet($this->dataSourceSetSituacaoFrequenciaParam, array('type' => 'success', $deletedRows)));
            $response->addDataSet(new DataSet($this->dataSourceSetSituacaoFrequencia, array('type' => 'success', $deletedRows)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteSituacaoFrequencia (DTO\Request\Row $row, DTO\Response $response) {
        try{
            $dataSet = new DataSet($this->dataSourceSetSituacaoFrequencia, array($row->getRow()));
            $deletedRows = $this->dataSourceManager->delete($dataSet);

            $response->addDataSet(new DataSet($this->dataSourceSetSituacaoFrequenciaParam, array('type' => 'success', $deletedRows)));
            $response->addDataSet(new DataSet($this->dataSourceSetSituacaoFrequencia, array('type' => 'success', $deletedRows)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteParametroSituacaoFrequencia (DTO\Request\Row $row, DTO\Response $response) {
        try{
            $dataSet = new DataSet($this->dataSourceSetParamSituacaoFrequencia, array($row->getRow()));
            $deletedRows = $this->dataSourceManager->delete($dataSet);

            $response->addDataSet(new DataSet($this->dataSourceSetParamSituacaoFrequencia, array('type' => 'success', $deletedRows)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}