<?php


/**
 * CAGED.
 *
 * @package Logic
 *
 * @author  thalesvalias
 */
class CAGED{

    CONST ALFANUMERICOS = '/[^a-zA-Z0-9]/';
    CONST ESTRUTURALEGAL = 'ESTRUTURALEGAL';
    CONST ESTRUTURAGERENCIAL = 'ESTRUTURAGERENCIAL';
	CONST VINCULO_CLT = "1";

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_mesCompetencia;
    private $_ambienteSalarial;
    private $_tpModalidadeSalarioOperador;
    private $_dtInicio;
    private $_dtFim;
    private $_dtAux;
    private $_dtRetroativa;
    private $_movimentacao;
    private $_estruturacao;
    private $_tpMovTransfM;
    private $_tpMovTransfH;
    private $_itemGrupo;
    private $_listaCalculoFolha;
    private $_evento;

    private $_registroB;
    private $_registroC;
    private $_registroX;
	private $_registroZ;
    private $_cagedMagnetico;
    private $_cagedMagneticoTrab;
    private $_atualizacao;
    private $_totalVinculos; // Total de vinculos por estrutura.
    private $_movimentacoesAdmissao = array();
    private $_movimentacoesRescisao = array();
    private $_dadoVinculo = array();

    private $_nrInscricao;
    private $_nrTpInscricao;

    private $_itemCalculo;
    private $_pessoa;
    private $_vinculo;
    private $_vinculoEmpreg;
    private $_alteracaoSalario;
    private $_ocupacao;
    private $_tpDemissao;
    private $_grauInstrucao;
    private $_escalaTrabalho;
    private $_raca;
    private $_condicaoFisica;
    private $_cbo;
    private $_enderecoParc;

    const TRANSFERENCIA_ENTRADA_CAGED = '70';
    const TRANSFERENCIA_SAIDA_CAGED = '80';
    const TIPO_TRANSFERENCIA_LEGAL = '1';
    const PRIMEIRA_DECLARACAO = '2';
    const ALTERACAO_DECLARACAO = '1';

    /**
     * Construtor.
     *
     * @param integer              $nrOrg          Numero da Organizacao.
     * @param integer              $nrOrgPadrao    Numero da Organizacao Padrao.
     * @param TkDateTime           $mesCompetencia Mes de competencia.
     * @param FpacagedmagnetEntity $cagedMagnetico Instancia de FpagfipmagnetEntity.
     * @param FpaformcagedEntity   $formulario     Formulario.
     * @param integer              $itensGrupo
     * @param integer              $listaNrCalculoFolha
     *
     * @author thalesvalias - 22/05/2012
     */
    public function __construct(
        $nrOrg, $nrOrgPadrao, TkDateTime $mesCompetencia, FpacagedmagnetEntity $cagedMagnetico, $formulario, $itensGrupo, $listaNrCalculoFolha
    ){

        $this->_nrOrg = $nrOrg;
        $this->_nrOrgPadrao = $nrOrgPadrao;
        $this->_mesCompetencia = $mesCompetencia;

        $this->_dtInicio = new TkDateTime($this->_mesCompetencia);
        $this->_dtFim = new TkDateTime($this->_mesCompetencia);
        $this->_dtFim->addMonth(1)->subDay(1);
        $this->_dtAux = new TkDateTime();

        $this->_itemCalculo = new FpaitecalcfolhaEntity();
        $this->_pessoa = new GpepessoaEntity(null, $this->_nrOrg, $this->_mesCompetencia);
        $this->_vinculo = new GpevinculomEntity();
        $this->_vinculoEmpreg = new GpevinculoempregEntity();
        $this->_ocupacao = new GpeOcupacaomEntity(null, $this->_nrOrg, $this->_mesCompetencia);
        $this->_tpDemissao = new FpaTpdemissaoEntity(null, $this->_nrOrgPadrao, $this->_nrOrg);
        $this->_movimentacao = new GpemovimentacaoEntity();
        $this->_tpMovTransfM = new GpetpmovtransfmEntity(
            null, $this->_nrOrgPadrao, $this->_nrOrg, $this->_mesCompetencia
        );
        $this->_tpMovTransfH = new GpetpmovtransfhEntity(
            null, $this->_nrOrgPadrao, $this->_nrOrg, $this->_mesCompetencia
        );
        $this->_alteracaoSalario = new GpealtesalarioEntity(null, $this->_nrOrg);
        $this->_grauInstrucao = new GpeGrauinstrEntity(null, $this->_nrOrg, $this->_nrOrgPadrao);
        $this->_escalaTrabalho = new GpeescalatrabmEntity();
        $this->_raca = new GpeRacapessoaEntity(null, $this->_nrOrgPadrao, $this->_nrOrg);
        $this->_condicaoFisica = new GpeCondfispesEntity(null, $this->_nrOrg, $this->_nrOrgPadrao);
        $this->_cbo = new GpeCboEntity();
        $this->_enderecoParc = new EnderecoparcEntity();

        $this->_itemGrupo = $itensGrupo;
        $this->_listaCalculoFolha = $listaNrCalculoFolha;
        $this->_evento = new FpaeventomEntity(null, $this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia);
        $this->_estruturacao = new Estruturacao($this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia);

        $this->_cagedMagnetico = $cagedMagnetico;
        $this->_cagedMagneticoTrab = new FpacagedmagnetEntity();

        $this->_registroB = new RegistroBCAGED();
        $this->_registroC = new RegistroCCAGED();
        $this->_registroX = new RegistroXCAGED();
		$this->_registroZ = new RegistroZCAGED();

        $this->_inicializaVariaveis($formulario);
    }


    private function _inicializaVariaveis($formulario){

        $this->_estruturacao->carregaHierarquias($formulario->getNrhierarqorg());
        $this->_estruturacao->carregaTipoHierarquia($formulario->getNrhierarqorg());
        $this->_atualizacao = $formulario->getIdatualizacao();

        $this->_movimentacoesAdmissao = array(10, 20, 25, 35, 70);
        $this->_movimentacoesRescisao  = array(31, 32, 40, 43, 45, 50, 60, 80);
        $this->_totalVinculos = array();
    }

    
    /* Cria os registros CAGED e armazena na base (B,C,X).
     * @param GpevinculomEntity $vinculo       Instancia de GpevinculomEntity.
     * @param string            $tpInscHierarq Tipo de Inscricao da Hierarquia.
     * @return void
     * @author thalesvalias - 22/05/2012
     */

    public function geraCAGED(GpevinculomEntity $vinculo, $tpInscHierarq){
        
        $this->_vinculo = $vinculo;
        $this->_dtAux->set($this->_dtInicio);

        // LEONARDOCARVALHO - 03/10/2014 - ALTERADO PARA BUSCAR TODAS AS MOVIMENTAÇÕES DO VÍNCULO
        // $estruturaResp = Gpemovimentacao::getMovimentacao($this->_nrOrg, \FPA::getTipoEstruturaLegalOrg(), $vinculo->getNrvinculom());
        $estruturaResp = Gpemovimentacao::getMovimentacoes($this->_nrOrg, \FPA::getTipoEstruturaLegalOrg(), $vinculo->getNrvinculom());
        if ($estruturaResp) {
            // LEONARDOCARVALHO - 03/10/2014 - PERCORRE TODAS AS MOVIMENTAÇÕES
            foreach ($estruturaResp as $estrutura) {
                $this->_defineTipoInscricao($tpInscHierarq, $estrutura);

                $this->_dtAux->set($estrutura->getDtinimoviment());

                $tipoAtualizacao = $this->_atualizacao;

                // ALTERAÇÃO: O TIPO DE ATUALIZAÇÃO FOI INSERIDO COMO DEFAULT '0 - NORMAL' NA MOVIMENTACAO - leonardocarvalho 07/08/2014
                // if ($tipoAtualizacao === '0') {
                //     $tipoAtualizacao = null;
                // }

                if ($this->_dtAux->isBetween($this->_dtInicio, $this->_dtFim)) {

                    if (($estrutura->getIdatualizaacerto() === $tipoAtualizacao)){

        //              calcula a bases para o caged (salario mensal).
                        $this->_calculaBasesCaged();

                        $this->_preparaRegistroB($estrutura->getEstrutura());

        //              retorna o cargo definido para o vinculo
                        $ocupacao = $this->_ocupacao->retornaOcupacao($this->_vinculo->historico->getNrcargo());

                        // verirfica data retroativa
                        $this->_dtRetroativa = $estrutura->getDtmovimentretro();


        //              verifica o tipo de atualizacao
                        if ($tipoAtualizacao !== '0') {
                            // LEONARDOCARVALHO - VERIFICA SE A DATA RETROATIVA DO MOVIMENTO FOI INFORMADA.
                            if ($this->_dtRetroativa != '') {
                                if ($this->_dtInicio->isLater($this->_dtRetroativa)) {
                                    //                      gera o registro X
                                    $this->_preparaRegistroX($estrutura, $ocupacao, $vinculo);
                                }
                            } else {
                                tk_warning('Data Retroativa do Movimento não foi Informada.');
                            }

                        } else {
        //                  se nao existir atualizacao gera o registro C

                            // Verifica se o vínculo possui PIS/PASEP cadastrado, caso contrário o mesmo não será gerado no arquivo. leonardocarvalho 05/10/2013
                            $pessoa = $this->_preparaPessoa($this->_vinculo->getNrpessoa());
                            if((bool)$pessoa->historico->getNrpispaseppes()){
                                if ((bool)$pessoa->historico->getNrcpfpessoa()) {
                                    $this->_preparaRegistroC($estrutura, $ocupacao);
                                } else {
                                    tk_warning('Pessoa '.$pessoa->historico->getNmpessoa().' não foi gerada pois CPF não foi informado.');
                                }
                            }else{
                                tk_warning('Pessoa '.$pessoa->historico->getNmpessoa().' não foi gerada pois PIS/PASEP não foi informado. '.$pessoa->historico->getNrpispaseppes());
                            };


                        }
                    }else{
                        tk_abort('Movimentação do vínculo '.$this->_vinculo->getNmvinculom().' não possui tipo de atualização.');
                    }
                }

            }
        }
//
////        carrega todas as movimentacoes legais do vinculo ate a data de competencia.
//        $this->_vinculo = $vinculo;
//        $movimentacoes = $this->_estruturacao->getMovimentacoesVinculoPorTipo($this->_vinculo->getNrvinculom(),
//            self::TIPO_TRANSFERENCIA_LEGAL);
//
////        pega o dia anterior a data de inicio
//        $this->_dtAux->set($this->_dtInicio);
//
//        $i = 1;
//        while($i <= count($movimentacoes)){
//
//            $movimentacao = current($movimentacoes);
//
////            data de inicio da movimentacao.
//            $this->_dtAux->set($movimentacao->getDtinimoviment());
//
////            pega o codigo CAGED da movimentacao.
//            $nrTipoMovimentacao = $movimentacao->getNrtpmovtransfm();
//            $tipoMovimentacao = $this->_tpMovTransfM->retornaTpMovTransf($nrTipoMovimentacao);
//            if (is_object($tipoMovimentacao) === true) {
//                $codCaged = $tipoMovimentacao->historico->getCodcaged();
//            } else {
//                $codCaged = null;
//            }
//
//
//            $intervaloCompetencia = is_null($this->_dtFim ) ?
//                $this->_dtAux->isEarlier($this->_dtInicio) :
//                $this->_dtAux->isBetween($this->_dtInicio, $this->_dtFim);
//
//
//            $this->_dtRetroativa = $movimentacao->getDtmovimentretro();
//            $tipoAtualizacao = $this->_atualizacao;
//
////            retornar a estrutura superior para gerar o registro B.
////            $estruturaResp = $this->_estruturacao->getEstruturaSuperior($movimentacao);
//            $estruturaResp = Gpemovimentacao::getMovimentacao(
//                $this->_nrOrg, \FPA::getTipoEstruturaLegalOrg(), $vinculo->getNrvinculom()
//            );
//
////            padroniza o tipo de atualizacao. Caso seja 0 (normal), recebe nulo (pois no formcaged eh 0 e na movimentacao eh nulo).
//            if ($tipoAtualizacao === '0') {
//                $tipoAtualizacao = null;
//            }
//
////            apenas considera a movimentacao que possue estrutura igual a do formulário.
//            if ($estruturaResp !== null) {
//
//                if ($tpInscHierarq === RegistroCAGED::CNPJ) {
////                    $this->_nrInscricao = $estruturaResp->historico->getCdcnpjestrut();
//                    $this->_nrInscricao = $estruturaResp->getEstrutura()->getHistorico()->getCdcnpjestrut();
//                    $this->_nrTpInscricao = RegistroCAGED::CNPJ;
//                } elseif ($tpInscHierarq === RegistroCAGED::CEI) {
////                    $this->_nrInscricao = $estruturaResp->historico->getCdceiestrut();
//                    $this->_nrInscricao = $estruturaResp->getEstrutura()->getHistorico()->getCdceiestrut();
//                    $this->_nrTpInscricao = RegistroCAGED::CEI;
//                }
//
//                $this->_consolidaTotalVinculosPorEstrutura($codCaged);
//
////                considera apenas as movimentacoes que estiverem no intervalo de competencia.
//                if ($intervaloCompetencia === true) {
//
////                    inclui movimentacao de saida.
//                    if ($codCaged === self::TRANSFERENCIA_ENTRADA_CAGED) {
//                        tk_warning('1');
////                        pega a movimentacao anterior, pois a movimentacao a ser incluida deve pegar essa estrutura,
////                        alem de outros atributos.
//                        $movimentacaoAnterior = next($movimentacoes);
//                        $movimentacaoSaida = $this->_incluiMovimentacaoSaida($movimentacaoAnterior);
//                        if ($movimentacaoSaida !== false) {
//                            $movimentacoes[] = $movimentacaoSaida;
//                            $movimentacaoSaida->persist();
//                        }
//
////                        volta para a movimentacao original.
//                        $movimentacao = prev($movimentacoes);
//                    }
//
////                    gera o CAGED apenas se o tipo de atualizacao da movimentacao for igual ao do formcaged
//                    if ($movimentacao->getIdatualizaacerto() === $tipoAtualizacao) {
//
////                        calcula a bases para o caged (salario mensal).
//                        $this->_calculaBasesCaged();
//
//                        $this->_preparaRegistroB($estruturaResp, $this->_vinculo->getNrvinculom());
//
////                        retorna o cargo definido para o vinculo
//                        $ocupacao = $this->_ocupacao->retornaOcupacao($this->_vinculo->historico->getNrcargo());
//
////                        verifica o tipo de atualizacao
//                        if ($tipoAtualizacao !== null) {
//
////                            verifica se existe data retroativa
//                            if (empty($this->_dtRetroativa) === false) {
//                                if ($this->_dtInicio->isLater($this->_dtRetroativa)) {
//
////                                    gera o registro X
//                                    $this->_preparaRegistroX($movimentacao, $ocupacao);
//                                }
//                            }
//                        } else {
////                            se nao existir atualizacao gera o registro C
//                            $this->_preparaRegistroC($movimentacao, $ocupacao);
//                        }
//                    }
//                }/*else{
//                    tk_warning('Movimentacao nao encontrada ou fora da data de competencia para o vinculo '
//                        .$this->_vinculo->getNrvinculom());
//                }*/
//            }/*else{
//                tk_warning('Tipo de estrutura nao parametrizada ou com parametrizacao diferente da selecionada
//                no formulario para a movimentacao '.$movimentacao->getNrmovimentacao());
//            }*/
//
//            next($movimentacoes);
//            $i++;
//        }
    }


    /*
     * Consolida o total de vinculos ativos por estrutura ate o primeiro dia da competencia.
     *
     * @param integer $codigoCaged Instancia de GpevinculomEntity, referente a movimentacao anterior a de entrada.
     *
     * @return void
     *
     * @author thalesvalias - 04/06/2012
     */
    private function _consolidaTotalVinculosPorEstrutura($codigoCaged) {

        // Verifica se a data de inicio da movimentacao eh menor que o primeiro dia do mes de competencia. Se sim, consolida normalmente.
        if ($this->_dtAux->isEarlier($this->_dtInicio) === true) {
            $this->_consolidaTotalVinculosTipoMovimentacao($codigoCaged);

         // Verifica se a data de inicio da movimentacao eh igual ao primeiro dia do mes de competencia.
         } else if ($this->_dtAux->equals($this->_dtInicio) === true) {

            // Verifica se possui data retroativa. Caso sim, consolida normalmente. Caso contrario, soma + 1 em caso de admissao e 0 em caso de rescisao.
            if (empty($this->_dtRetroativa) === false) {
                $diaPrimeiro = false;
            } else {
                $diaPrimeiro = true;
            }

            $this->_consolidaTotalVinculosTipoMovimentacao($codigoCaged, $diaPrimeiro);
        }
    }


    /**
     * Consolida o total de vinculos ativos por estrutura ate o primeiro dia da competencia. Decresce ou acresce valor de acordo com
     * o tipo de movimentacao de transferencia informado (codigo caged). Em caso de dia primeiro, rescisao equivale a 0.
     *
     * @param integer $codigoCaged Instancia de GpevinculomEntity, referente a movimentacao anterior a de entrada.
     * @param Boolean $diaPrimeiro Informa se a data de inicio de movimentacao eh dia 01.
     *
     * @return void
     *
     * @author thalesvalias - 05/06/2012
     */
    private function _consolidaTotalVinculosTipoMovimentacao($codigoCaged, $diaPrimeiro = false) {
        // Se for movimentacao de admissao, acresce um do total. Se rescisao, descresce um.
        if (in_array($codigoCaged, $this->_movimentacoesAdmissao) === true) {
            $this->_totalVinculos[$this->_nrInscricao]++;
        } else if (in_array($codigoCaged, $this->_movimentacoesRescisao) === true) {
            if ($diaPrimeiro === false) {
                if (isset($this->_totalVinculos[$this->_nrInscricao])) {
                    $this->_totalVinculos[$this->_nrInscricao]--;
                } else {
                    // Inicia com -1, caso esteja nulo.
                    $this->_totalVinculos[$this->_nrInscricao] = -1;
                }
            }
        }
    }


    /**
     * Inclui uma movimentacao de saida referente a uma movimentacao de entrada.
     *
     * @param GpemovimentacaoEntity $movimentacaoAnterior Instancia de GpevinculomEntity, referente a movimentacao anterior a de entrada.
     *
     * @return false||GpemovimentacaoEntity
     *
     * @author thalesvalias - 02/06/2012
     */
    private function _incluiMovimentacaoSaida($movimentacaoAnterior) {

        $diaPrimeiro = $this->_mesCompetencia->equals($this->_dtAux);
        $estruturaMovimentacaoAnterior = $movimentacaoAnterior->getNrestruturam();
        $nrTipoTransferenciaAnterior = $movimentacaoAnterior->getNrtipotransfer();
        $nrTipoEstruturaAnterior =  $movimentacaoAnterior->getNrtipoestrutura();

        $tipoMovimentacaoTransferencia = $this->_tpMovTransfH->retornaTipoMovTransfHistoricoPorCaged(
            self::TRANSFERENCIA_SAIDA_CAGED
        );
        $nrTipoMovimentacaoTransferencia = $tipoMovimentacaoTransferencia->getNrtpmovtransfm();

        $dtInicioMovimento = $this->_dtAux->toString();
        $dtAnteriorInicioMovimento = $this->_dtAux->subDay(1)->toString();
        // Nesse caso a data de fim de movimentacao eh igual a data de inicio da transferencia de saida.
//        $dtFimMovimento = $dtAnteriorInicioMovimento;

        $dataRetroativa = null;
        if (empty($this->_dtRetroativa) === false) {
//            $this->_dtAux->set($this->_dtRetroativa);
            $dataRetroativa = $dtAnteriorInicioMovimento;
            $atualizacao = GpemovimentacaoEntity::IDATUALIZAACERTO_INCLUIR;
        } else {
            if ($diaPrimeiro === true) {
                $dataRetroativa = $dtAnteriorInicioMovimento;
                $atualizacao = GpemovimentacaoEntity::IDATUALIZAACERTO_INCLUIR;
            } else {
                $atualizacao = null;
                $dtInicioMovimento = $dtAnteriorInicioMovimento;
            }

        }

        $verificaMovimentacaoSaida = $this->_estruturacao->verificaExistenciaMovimentacao(
            $nrTipoMovimentacaoTransferencia, $dtInicioMovimento, $dataRetroativa
        );

        if ($verificaMovimentacaoSaida === false) {
            $mov = $this->_estruturacao->incluiMovimentacaoVinculo(
                $nrTipoTransferenciaAnterior, $this->_vinculo->getNrvinculom(), $estruturaMovimentacaoAnterior, $dtInicioMovimento,
                $nrTipoEstruturaAnterior, $nrTipoMovimentacaoTransferencia, $dataRetroativa, $atualizacao, null//, $dtFimMovimento
            );

            return $mov;
        } else {
            return false;
        }

    }


    /**
     * Prepara para a geracao do Registro B.
     *
     * @param integer $estrutura     Numero da estrutura superior referente ao vinculo.
     * @param integer $nrVinculo     Numero do vinculo.
     *
     * @return void
     *
     * @author thalesvalias - 22/05/2012
     *         leonardocarvalho - 23/10/2014 - alterado assinatura da função para público
     */
    public function _preparaRegistroB($estrutura){

        $posicao = $this->_cagedMagnetico->procuraRegistroLista(
            $this->_nrInscricao, RegistroBCAGED::REGISTRO_B
        );

        if ($posicao === false) {

            $endereco = $this->_estruturacao->determinaEnderecoEstrutura($estrutura);
            $estrutura->endereco = $endereco;
            $this->_registroB->setEstrutura($estrutura);
            $this->_registroB->setTpInscricao($this->_nrTpInscricao);
            $this->_registroB->setNrInscricao($this->_nrInscricao);
            $this->_registroB->setPrimeiraDeclaracao(self::PRIMEIRA_DECLARACAO);
            $this->_registroB->setAlteracaoDeclaracao(self::ALTERACAO_DECLARACAO);

            // leonardocarvalho - 27/10/2014 - retrocede a competencia ao último dia do mês anterior
            $dtCompMesAnt = new TkDateTime($this->_mesCompetencia);
            $dtCompMesAnt->subMonth(1);

            // retorna o total de vinculos ativos na estrutura no mês da competencia
			// leonardocarvalho - 06/05/2015 - considera somente vinculos do tipo CLT
            $vinculos = $this->_vinculo->retornaVinculosAtivosMesCompetencia(
                $this->_nrOrg, $dtCompMesAnt, $this->_dsInicio, $this->_dsFim, $this->_dsOrdem, false,
                $estrutura->getNrestruturam(), false
            );
			$countTotalVinculosCLT = 0;
			foreach ($vinculos as $vinculo) {
				if ($vinculo->getNrtipovinculom() == self::VINCULO_CLT) {
					$countTotalVinculosCLT++;
				}
			}
            $totalVinculosEstrutura = $countTotalVinculosCLT;
            $this->_registroB->setTotalEmpregados($totalVinculosEstrutura);

            // Monta a linha registro B.
            $this->_registroB->geraRegistroB();

            // Insere novo registro tipo B.
            $this->_cagedMagnetico->insereRegistroLista(
                $this->_nrOrg, $this->_mesCompetencia, RegistroBCAGED::REGISTRO_B, $this->_nrTpInscricao,
                $this->_nrInscricao, $this->_registroB->getLinhaRegistro(), null, null, null, $totalVinculosEstrutura
            );

            // Limpa o registro B.
            $this->_registroB->setLinhaRegistro();
        }
    }


    /**
     * Prepara para a geracao do Registro C.
     *
     * @param GpemovimentacaoEntity $movimentacao Instancia de GpemovimentacaoEntity.
     * @param GpeOcupacaomEntity    $ocupacao     Instancia de GpeOcupacaomEntity.
     *
     * @return void
     *
     * @author thalesvalias - 22/05/2012
     */
    private function _preparaRegistroC(
        GpemovimentacaoEntity $movimentacao, GpeOcupacaomEntity $ocupacao
    ){
        $this->_preparaVinculo($movimentacao, $ocupacao);
        $pessoa = $this->_preparaPessoa($this->_vinculo->getNrpessoa());

        $this->_registroC->setVinculo($this->_vinculo);
        $this->_registroC->setPessoa($pessoa);
        $this->_registroC->setTpInscricao($this->_nrTpInscricao);
        $this->_registroC->setNrInscricao($this->_nrInscricao);

        // Registro C.
        $this->_registroC->geraRegistroC();

        // PisPasep.
        $nrPisPasepCi = $pessoa->historico->getNrpispaseppes();

        // Persiste registro linha C.
        $this->_cagedMagnetico->persisteRegistro(
            $this->_nrOrg, $this->_mesCompetencia, RegistroCCAGED::REGISTRO_C, $this->_nrTpInscricao,
            $this->_nrInscricao, $this->_registroC->getLinhaRegistro(), $this->_vinculo->getNrvinculom(),
            $this->_vinculo->getDtadmissaovinc(), $nrPisPasepCi, $movimentacao->getNrtpmovtransfm(), $this->_vinculo->getDtrescisaovinc()
        );

        // Limpa o registro C.
        $this->_registroC->setLinhaRegistro();
    }


    /**
     * Prepara para a geracao do Registro X.
     *
     * @param GpemovimentacaoEntity $movimentacao Instancia de GpemovimentacaoEntity.
     * @param GpeOcupacaomEntity    $ocupacao     Instancia de GpeOcupacaomEntity.
     *
     * @return void
     *
     * @author thalesvalias - 06/06/2012
     */
    private function _preparaRegistroX(
        GpemovimentacaoEntity $movimentacao, GpeOcupacaomEntity $ocupacao, GpevinculomEntity $vinculo
    ){
        $this->_preparaVinculo($movimentacao, $ocupacao, true);
        $pessoa = $this->_preparaPessoa($vinculo->getNrpessoa());

        $this->_dtAux->set($this->_dtRetroativa);

        $this->_registroX->setCompetenciaRetroativa($this->_dtAux);
        $this->_registroX->setIdAtualizacao($movimentacao->getIdatualizaacerto());
        $this->_registroX->setTpInscricao($this->_nrTpInscricao);
        $this->_registroX->setNrInscricao($this->_nrInscricao);
        $this->_registroX->setPessoa($pessoa);
        $this->_registroX->setVinculo($vinculo);

        // Registro X.
        $this->_registroX->geraRegistroX();

        // PisPasep.
        $nrPisPasepCi = $pessoa->historico->getNrpispaseppes();

        // Persiste registro linha X.
        $this->_cagedMagnetico->persisteRegistro(
            $this->_nrOrg, $this->_mesCompetencia, RegistroXCAGED::REGISTRO_X, $this->_nrTpInscricao,
            $this->_nrInscricao, $this->_registroX->getLinhaRegistro(), $vinculo->getNrvinculom(),
            $vinculo->getDtadmissaovinc(), $nrPisPasepCi, $movimentacao->getNrtpmovtransfm()
        );

        // Limpa o registro X.
        $this->_registroX->setLinhaRegistro();
    }


    private function _preparaVinculo(GpemovimentacaoEntity $movimentacao, GpeOcupacaomEntity $ocupacao, $registroXcaged = false){

        $this->_vinculo->salario =  (float)$this->_dadoVinculo[RegistroCAGED::CAGED_SALARIO_MENSAL];

        // Quantidade de horas trabalhadas.
        $nrEscalaTrabalho = $this->_vinculo->historico->getNrescalatrabm();
        $escalaTrabalho = $this->_escalaTrabalho->retornaEscala($nrEscalaTrabalho, $this->_nrOrg);
        if (is_object($escalaTrabalho) === true) {
            $this->_vinculo->escalaSemanal = $escalaTrabalho->historico->getQthrsemesctrabh();
        } else {
            $this->_vinculo->escalaSemanal = null;
        }

//        @todo Verificar codigo CBO.
        if (is_object($ocupacao) === true){
            $nrCbo = $ocupacao->historico->getNrcbo();
            $cbo = $this->_cbo->retornaCbo($ocupacao->historico->getNrcbo());

            if($cbo){
                $this->_vinculo->cbo = $cbo->getCdrais();
            }else{
                $this->_vinculo->cbo = null;
                tk_warning('Vinculo ' . $this->_vinculo->getNrvinculom() . ' não possui CBO.');
            }
        } else {
            tk_warning('Cargo nao definido para o vinculo '.$this->_vinculo->getNrvinculom());
        }

        // Codcaged do tipo de movimentacao.
        $nrTipoMovimentacao = $movimentacao->getNrtpmovtransfm();
        $tipoMovimentacao = $this->_tpMovTransfM->retornaTpMovTransf($nrTipoMovimentacao);
        if (is_object($tipoMovimentacao) === true) {
            $this->_vinculo->tipoMovimentacao = $tipoMovimentacao->historico->getCodcaged();
        } else {
            $this->_vinculo->tipoMovimentacao = null;
        }

        // Codcaged do vinculo empregaticio.
        $nrVinculoEmpregaticio = $this->_vinculo->getNrvinculoempreg();
        $vinculoEmpregaticio = $this->_vinculoEmpreg->retornaVinculoEmpregaticio($nrVinculoEmpregaticio);
        if (is_object($vinculoEmpregaticio) === true) {
            $this->_vinculo->empregaticio = $vinculoEmpregaticio->getCodcaged();
        } else {
            $this->_vinculo->empregaticio = null;
        }

//        @todo verificar
        // LEONARDOCARVALHO - ADICIONADA CONDIÇÃO PARA PROCESSAR AS DATAS SOMENTE QUANDO NÃO FOR DO REGISTRO X
        if ($registroXcaged == false) {
            if ($movimentacao->getNrtpmovtransfm()) {
                $this->_processaDataAdmissaoRescisaoMovimentacao($movimentacao, $tipoMovimentacao->historico->getCodcaged());
            }
        }
    }


    private function _preparaPessoa($nrPessoa){

        $pessoa = $this->_pessoa->retornaPessoaAtual($nrPessoa);

        // Codcaged do grau de instrucao.
        $nrGrauInstrucao = $pessoa->historico->getNrgrauinstr();
        $grauInstrucao = $this->_grauInstrucao->retornaGrauInstrucaoPorCodigo($nrGrauInstrucao);
        if (is_object($grauInstrucao) === true) {
            $pessoa->instrucao = $grauInstrucao->getCodcaged();
        } else {
            $pessoa->instrucao = null;
        }

        // Codcaged da raca.
        $nrRaca = $pessoa->historico->getNrracapessoa();
        $racaPessoa = $this->_raca->retornaRacaPessoaPorCodigo($nrRaca);
        if (is_object($racaPessoa) === true) {
            $pessoa->raca = $racaPessoa->getCodcaged();
        } else {
            $pessoa->raca = null;
        }

        // Codcaged da condicao fisica.
        $nrCondicaoFisica = $pessoa->historico->getNrcondfispes();
        $condicaoFisica = $this->_condicaoFisica->retornaCondicaoFisicaPorCodigo($nrCondicaoFisica);
        if (is_object($condicaoFisica) === true) {
            $pessoa->condicaoFisica = $condicaoFisica->getCodcaged();
            if ($pessoa->condicaoFisica !== null) {
                $pessoa->portadorDeficiencia = 1;
            } else {
                $pessoa->portadorDeficiencia = 2;
            }
        } else {
            $pessoa->condicaoFisica = null;
            $pessoa->portadorDeficiencia = 2;
        }

        // CEP do trabalhador.
        $nrParceiroNegocio = $pessoa->getNrparcnegocio();
        // @todo passar constantes de tipo de endereco para TipoenderecoEntity (atualmente em Estruturacao).
        $enderecoParc = $this->_enderecoParc->retornaEnderecoParceiroPorTipo(
            $this->_nrOrg, $nrParceiroNegocio, Estruturacao::ENDERECO_PRINCIPAL
        );
        if (is_object($enderecoParc) === true) {
            $pessoa->cep = $enderecoParc->getNrcependereco();
        } else {
            $pessoa->cep = null;
        }

        return $pessoa;
    }


    private function _processaDataAdmissaoRescisaoMovimentacao(GpemovimentacaoEntity $movimentacao, $codigoCaged) {

//        $dataRetroativa = $movimentacao->getDtmovimentretro();
        $dataInicioMov = $movimentacao->getDtinimoviment();

        if (in_array($codigoCaged, $this->_movimentacoesAdmissao) === true) {
            if (empty($this->_dtRetroativa) === false) {
                $this->_vinculo->setDtadmissaovinc($this->_dtRetroativa);
            } else {
                $this->_vinculo->setDtadmissaovinc($dataInicioMov);
            }
            $this->_vinculo->setDtrescisaovinc(null);
        } else if (in_array($codigoCaged, $this->_movimentacoesRescisao) === true) {
            if (empty($this->_dtRetroativa) === false) {
                $this->_vinculo->setDtrescisaovinc($this->_dtRetroativa);
            } else {
                $this->_vinculo->setDtrescisaovinc($dataInicioMov);
            }

            $movimentacaoAnterior = $this->_movimentacao->retornaMovimentacaoAnterior(
                $this->_mesCompetencia, $this->_nrOrg, $movimentacao->getNrvinculom(),
                $dataInicioMov, $this->_movimentacoesAdmissao
            );

            $dtMovimentacaoRetroativa = $movimentacaoAnterior['DTMOVIMENTRETRO'];
            $dtMovimentacaoInicio = $movimentacaoAnterior['DTINIMOVIMENT'];
            if (empty($dtMovimentacaoRetroativa) === false) {
                $this->_vinculo->setDtadmissaovinc($dtMovimentacaoRetroativa);
            } else {
                $this->_vinculo->setDtadmissaovinc($dtMovimentacaoInicio);
            }
        }
    }


    private function _calculaValorItemGrupo($referencia){
        $valor = 0;
        $itemGrupo = $this->_itemGrupo->getItem($referencia);
        $nrTipoIncidencia = $itemGrupo->getNrtipoincide();
        $valor = $this->_itemCalculo->calculaIncidencias($nrTipoIncidencia, $this->_mesCompetencia, $this->_evento);
        return $valor;
    }


    public function modificaTotalVinculos() {

        foreach ($this->_totalVinculos as $nrInscricao => $valor) {
            // Verifica se o valor e negativo. Se for, passa a ser 0.
            if ($valor < 0) {
                $valor = 0;
                tk_warning("Contagem de vinculos por estrutura deu negativo para a estrutura de CNPJ $nrInscricao");
            }

            // Testa a posicao diferente de 0, pois ao ser declarado o vetor cria esta posicao e a mesma nao eh utilizada.
            if ($nrInscricao !== 0) {

                $posRegistro = $this->_cagedMagnetico->procuraRegistroLista($nrInscricao, RegistroBCAGED::REGISTRO_B);
                if ($posRegistro !== false) {

                    $registroMag = $this->_cagedMagnetico->getItem($posRegistro);

                    if (is_object($registroMag) === true) {
                        // Alteracao do numero total de vinculos.
                        $linha = $this->_registroB->alteraDado(
                            $registroMag->getDslinharegistro(), (string)$valor, (139 -1), 5, RegistroBCAGED::CARACTER_ZERO,
                            RegistroBCAGED::PREENCHE_ESQUERDA
                        );
                        $registroMag->setDslinharegistro($linha);
                    }
                }
            }
        }

    }


    public function modificaSequencia($linha, $tipoRegistro, $sequencia){

        switch ($tipoRegistro){
        case RegistroBCAGED::REGISTRO_B:
            $linha = $this->_registroB->alteraDado(
                $linha, (string)$sequencia, (17 -1), 5, RegistroBCAGED::CARACTER_ZERO, RegistroBCAGED::PREENCHE_ESQUERDA
            );
            break;

        case RegistroCCAGED::REGISTRO_C:
            $linha = $this->_registroC->alteraDado(
                $linha, (string)$sequencia, (17 -1), 5, RegistroCCAGED::CARACTER_ZERO, RegistroCCAGED::PREENCHE_ESQUERDA
            );
            break;

        case RegistroXCAGED::REGISTRO_X:
            $linha = $this->_registroX->alteraDado(
                $linha, (string)$sequencia, (17 -1), 5, RegistroXCAGED::CARACTER_ZERO, RegistroXCAGED::PREENCHE_ESQUERDA
            );
            break;
        }

        return $linha;

    }
    
    /**
     * Retrocede os registros da Org. no mes de competencia.
     *
     * @return void
     *
     * @author thalesvalias - 22/05/2012
     */
    public function retroceder(){
        $this->_cagedMagnetico->retroceder($this->_nrOrg, $this->_mesCompetencia);
    }


    private function _calculaBasesCaged(){

       // carregar todos os itens de calculo.
        foreach ($this->_listaCalculoFolha as $nrCalculo) {

            $this->_dadoVinculo = array();

            $this->_itemCalculo->retornaItensPorVinculo($this->_nrOrg, $this->_vinculo->getNrvinculom(), $nrCalculo);

            $this->_calculaSalarioMensal();

        }
    }


    private function _calculaSalarioMensal(){

         if ($this->_ambienteSalarial === GpeParamoperEntity::IDAMBIENTESAL_REAL) {
            $nrModalidade = $this->_vinculo->historico->getNrtpmodalidsal();
        } else {
            $nrModalidade = $this->_tpModalidadeSalarioOperador;
        }

        // Carrega as alteracoes salariais.
        $carregou = $this->_alteracaoSalario->carregaAlteracaoSalario(
            $this->_mesCompetencia, $this->_vinculo->getNrvinculom(), $nrModalidade
        );

        if ($carregou === true) {
            $valor = $this->_alteracaoSalario->getSalario();
           // $tipoSalario = $this->_alteracaoSalario->getTipoSalario();
        } else {
            $msg = 'Nao encontrado valor de salario para o vinculo '. $this->_vinculo->getNrvinculom().'.';
            $msg .= ' Verificar Tipo de Modalidade de Salarial ou a Competencia';
            tk_warning($msg);

            return;
        }


       // $valor = $this->_calculaValorItemGrupo(RegistroCAGED::CAGED_SALARIO_MENSAL);
        $this->_consolidaDadosVinculo(RegistroCAGED::CAGED_SALARIO_MENSAL, $valor);
    }


    private function _consolidaDadosVinculo($referencia, $valor) {
        $this->_dadoVinculo[$referencia] = $valor;
    }

    public function setAmbienteSalarial($ambienteSalarial){
        $this->_ambienteSalarial = $ambienteSalarial;
    }

    public function setTpModalidadeSalarioOperador($tpModalidadeSalarioOperador){
        $this->_tpModalidadeSalarioOperador = $tpModalidadeSalarioOperador;
    }

    /**
     * @param $tpInscHierarq
     * @param \Gpemovimentacao|\GpemovimentacaoEntity $estruturaResp
     */
    private function _defineTipoInscricao($tpInscHierarq, GpemovimentacaoEntity $estruturaResp)
    {
        if ($tpInscHierarq === RegistroCAGED::CNPJ) {
            $this->_nrInscricao = preg_replace(
                self::ALFANUMERICOS, '',
                $estruturaResp->getEstrutura()->getHistorico()->getCdcnpjestrut()
            );
            $this->_nrTpInscricao = RegistroCAGED::CNPJ;
        } else {
            $this->_nrInscricao = preg_replace(
                self::ALFANUMERICOS, '',
                $estruturaResp->getEstrutura()->getHistorico()->getCdceiestrut()
            );

            $this->_nrTpInscricao = RegistroCAGED::CEI;
        }
    }

    /**
     * @param $nrTpInscricao
     * @author leonardocarvalho 23/10/2014
     */
    public function setTpInscricao($nrTpInscricao){
        $this->_nrTpInscricao = $nrTpInscricao;
    }

    /**
     * @param $nrInscricao
     * @author leonardocarvalho 23/10/2014
     */
    public function setNrInscricao($nrInscricao){
        $this->_nrInscricao = $nrInscricao;
    }

}